/*
 * File         : mpi_Status.c
 * Headerfile   : mpi_Status.h 
 * Author       : Sung-Hoon Ko, Xinying Li
 * Created      : Thu Apr  9 12:22:15 1998
 * Revision     : $Revision: 1.5 $
 * Updated      : $Date: 1999/10/06 21:39:47 $
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#include <mpi.h>
#include "mpi_Status.h"

jfieldID stathandleID;
extern jfieldID DatatypehandleID;
jfieldID sourceID, tagID,countID, indexID;

jmethodID handleConstructorID ;

/* jclass status_class ; */


/*
 * Class:     mpi_Status
 * Method:    init
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_init
  (JNIEnv *env, jclass jthis)
{
  stathandleID = (*env)->GetFieldID(env,jthis,"handle","J");

  sourceID     = (*env)->GetFieldID(env,jthis,"source","I");
  tagID        = (*env)->GetFieldID(env,jthis,"tag","I");
  indexID      = (*env)->GetFieldID(env,jthis,"index","I");

  handleConstructorID = (*env)->GetMethodID(env, jthis, "<init>", "()V");

  /* status_class = (*env)->NewGlobalRef(env, jthis) ; */
}

/*
 * Class:     mpi_Status
 * Method:    alloc
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_alloc
  (JNIEnv *env, jobject jthis) {
  MPI_Status *status = (MPI_Status*) malloc(sizeof(MPI_Status));

  (*env)->SetLongField(env, jthis, stathandleID, (jlong)status);
}

/*
 * Class:     mpi_Status
 * Method:    finalize
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_mpi_Status_finalize
  (JNIEnv *env, jobject jthis) {
   MPI_Status *status =
      (MPI_Status *)((*env)->GetLongField(env,jthis,stathandleID));

   free(status) ;
}

/*
 * Class:     mpi_Status
 * Method:    get_count
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_get_1count
  (JNIEnv *env, jobject jthis, jobject type)
{
   int count;

   MPI_Status *stat; /*shko*/

   stat=(MPI_Status *)((*env)->GetLongField(env,jthis,stathandleID)); /*shko*/

   /*
   stat.st_count=(*env)->GetIntField(env,jthis,countID);
   stat.MPI_SOURCE=(*env)->GetIntField(env,jthis,sourceID);
   stat.MPI_TAG=(*env)->GetIntField(env,jthis,tagID);
   */

   /* stat.MPI_ERROR=(*env)->GetIntField(env,jthis,errorID); */
   /*printf(" in STATUS count = %d \n",stat->count);*/

  MPI_Get_count( stat, 
       (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), 
       &count );
  /*  printf(" count = %d \n",count);*/

   return count;

}

/*
 * Class:     mpi_Status
 * Method:    Test_cancelled
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_mpi_Status_Test_1cancelled
  (JNIEnv *env, jobject jthis)
{
   int flag;
   MPI_Status *stat;  /*shko*/

   stat=(MPI_Status *)((*env)->GetLongField(env,jthis,stathandleID));/*shko*/
   /*
   stat.st_count=(*env)->GetIntField(env,jthis,countID);
   stat.MPI_SOURCE=(*env)->GetIntField(env,jthis,sourceID);
   stat.MPI_TAG=(*env)->GetIntField(env,jthis,tagID);
   */
   /* stat.MPI_ERROR=(*env)->GetIntField(env,jthis,errorID); */

   MPI_Test_cancelled(stat, &flag);
   if(flag==0)
        return JNI_FALSE;
    else
        return JNI_TRUE;
}

/*
 * Class:     mpi_Status
 * Method:    get_elements
 * Signature: (Lmpi/Datatype;)I
 */
JNIEXPORT jint JNICALL Java_mpi_Status_get_1elements
  (JNIEnv *env, jobject jthis, jobject type)       
{
   int count;
   MPI_Status *stat;

   stat=(MPI_Status *)((*env)->GetLongField(env,jthis,stathandleID));

   MPI_Get_elements( stat,
    (MPI_Datatype)((*env)->GetLongField(env,type,DatatypehandleID)), 
    &count );

   return count;
}

